// Priority.cpp : implementation file
//

#include "stdafx.h"
#include "Priority.h"
#include "Unavailable.h"
#include "dim.h"
#include "Colors.h"

// CPriority
#define DECLARE_BRUSH(pri) CBrush BRUSH_##pri(PriorityColors::GetBkColor(pri));

DECLARE_BRUSH(THREAD_PRIORITY_IDLE)
DECLARE_BRUSH(THREAD_PRIORITY_LOWEST)
DECLARE_BRUSH(THREAD_PRIORITY_BELOW_NORMAL)
DECLARE_BRUSH(THREAD_PRIORITY_NORMAL)
DECLARE_BRUSH(THREAD_PRIORITY_ABOVE_NORMAL)
DECLARE_BRUSH(THREAD_PRIORITY_HIGHEST)
DECLARE_BRUSH(THREAD_PRIORITY_TIME_CRITICAL)

#define INFO_ENTRY(x) { x, TRUE, PriorityColors::GetTextColor(x), &BRUSH_##x}

static const 
DisplayInfo DisplayInfoTable[] = {
 { UNAVAILABLE,                  FALSE, RGB(0,0,0),         NULL},
 INFO_ENTRY(THREAD_PRIORITY_IDLE),
 INFO_ENTRY(THREAD_PRIORITY_LOWEST),
 INFO_ENTRY(THREAD_PRIORITY_BELOW_NORMAL),
 INFO_ENTRY(THREAD_PRIORITY_NORMAL),
 INFO_ENTRY(THREAD_PRIORITY_ABOVE_NORMAL),
 INFO_ENTRY(THREAD_PRIORITY_HIGHEST),
 INFO_ENTRY(THREAD_PRIORITY_TIME_CRITICAL)
};

IMPLEMENT_DYNAMIC(CPriority, CStatic)
CPriority::CPriority()
   {
    SetPriority(UNAVAILABLE);
   }

CPriority::~CPriority()
{
}


BEGIN_MESSAGE_MAP(CPriority, CStatic)
    ON_WM_CTLCOLOR_REFLECT()
END_MESSAGE_MAP()



// CPriority message handlers


HBRUSH CPriority::CtlColor(CDC* pDC, UINT /*nCtlColor*/)
    {
     if(info == NULL)
        return NULL;
     if(!info->enable)
        { /* unavailable */
         EnableWindow(FALSE);
         return NULL;
        } /* unavailable */

     EnableWindow(TRUE);
     pDC->SetTextColor(info->text);
     pDC->SetBkMode(TRANSPARENT);
     return (HBRUSH)*info->brush;
    }

/****************************************************************************
*                           CPriority::SetPriority
* Inputs:
*       int pri: Desired priority
* Result: void
*       
* Effect: 
*       Selects the desired priority
****************************************************************************/

void CPriority::SetPriority(int pri)
    {
     info = NULL; 
     priority = pri;
     if(GetSafeHwnd() != NULL)
        Invalidate();

     for(int i = 0; i < DIM(DisplayInfoTable); i++)
        { /* scan table */
         if(priority == DisplayInfoTable[i].priority)
            { /* found it */
             info = &DisplayInfoTable[i];
             return;
            } /* found it */
        } /* scan table */

     ASSERT(FALSE); // should never get here!
    } // CPriority::SetPriority
