// affinityDlg.h : header file
//

#pragma once
#include "afxwin.h"
#include "afxcmn.h"
#include "Trace.h"
#include "idcombo.h"
#include "priority.h"
#include "maxthreads.h"
#include "graph.h"
#include "priorities.h"
#include "ProcessPriority.h"
#include "SmartDropdown.h"
#include "PosInfo.h"
#include "ChildProcess.h"
#include "ThreadInfo.h"

class GRAPHINFO;
class THREADINFO;

/*****************************************************************************
*                                    CaffinityDlg
*****************************************************************************/

// CaffinityDlg dialog
class CaffinityDlg : public CDialog
{
// Construction
public:
        CaffinityDlg(CWnd* pParent = NULL);     // standard constructor

// Dialog Data
        enum { IDD = IDD_AFFINITY_DIALOG };

        protected:
        virtual void DoDataExchange(CDataExchange* pDX);        // DDX/DDV support


// Implementation
protected: // Variables
        HICON m_hIcon;
        DWORD_PTR processAffinityMask;
        DWORD_PTR systemAffinityMask;
        //******************
        // thread management
        //******************
        long threadcount;
        long OffsetCount;
        BOOL Primary;

        //************************
        // Master/child management
        //************************
        HANDLE CreationMutex;
        CString configuration;
        ChildProcesses children;
        BOOL closing;
        void SelectRegistryConfig();
        void SelectConfiguration();

        //****************
        // dragging
        //****************
        int dxl;  // offset to left of mouse to right edge of left box
        int dxr;  // offset to right of mouse to left edge of right box

        //********************************
        // Command-line parameter handling
        //********************************
        CString GetCommandParameter(const CString & key);
        BOOL ParseCommandParameter(const CString & key, const CString & prefix, CString & result);
protected: // Registry watcher
        void DoWatch(HANDLE changeEvent);
        static UINT WatchConfigurationKey(LPVOID p);
        HANDLE StopWatchEvent; // used for program shutdown
        
protected: // methods
        BOOL AddGraph(const CRect & r, HWND owner);
        void DeleteGraph(HWND owner);
        CString GetConfiguration();
        void InitSecondaryInstance();
        BOOL InSplitterArea(CPoint pt);
        void LoadProfile();
        void Log(TraceEvent *);
        CString MakeConfigPrefix();
        CString MakeInstancePrefix();
        void PlotAllValues();
        void ResizeCopyRect();
        void ResizeGraphs();
        void RunAllWaiting();
        void SaveConfigurationName();
        void SaveProfile();
        void SaveThreadProfile(int n);
        void SetAllIterations(DWORD n);
        void SetGUIThreadPriority(int priority, const CString & name);
        void SetGUIThreadPriority(int priority, UINT id);
        BOOL SetPriorityClass(DWORD priority, const CString & name);
        void SetupAllWaiting();
        BOOL TryPlacement(CRect r, const CRect & Screen, STARTUPINFO & startup);
        BOOL TrySetCursor(UINT nHitTest);
        BOOL TryLButtonDown(CPoint pt);
        void updateControls();
        void CopyDataAddPoint(POSINFO * pos);
        void CopyDataAddGraphInfo(GRAPHINFO * info);
        void CopyDataAddThreadInfo(THREADINFO * info);

protected: // Event handlers
        // Generated message map functions
        virtual BOOL OnInitDialog();
        afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
        afx_msg void OnPaint();
        afx_msg HCURSOR OnQueryDragIcon();

        afx_msg void    OnAff1(UINT id);
        afx_msg void    OnAff2(UINT id);
        afx_msg void    OnAff3(UINT id);
        afx_msg void    OnAff4(UINT id);
        afx_msg void    OnAff5(UINT id);
        afx_msg void    OnAff6(UINT id);
        afx_msg void    OnAff7(UINT id);
        afx_msg void    OnAff8(UINT id);
        afx_msg LRESULT OnAnyRunning(WPARAM, LPARAM);
        afx_msg void    OnBnClickedAll();
        afx_msg void    OnBnClickedCopy();
        afx_msg void    OnBnClickedCopyall();
        afx_msg void    OnBnClickedCopyPriorityClass();
        afx_msg void    OnBnClickedCopyProgress();
        afx_msg void    OnBnClickedGo1();
        afx_msg void    OnBnClickedGo2();
        afx_msg void    OnBnClickedGo3();
        afx_msg void    OnBnClickedGo4();
        afx_msg void    OnBnClickedGo5();
        afx_msg void    OnBnClickedGo6();
        afx_msg void    OnBnClickedGo7();
        afx_msg void    OnBnClickedGo8();
        afx_msg void    OnBnClickedLaunch();
        afx_msg void    OnBnClickedResetAll();
        afx_msg void    OnBnClickedStop();
        virtual void    OnCancel();
        afx_msg void    OnCbnSelendokConfiguration();
        afx_msg void    OnCbnSelendokGUI();
        afx_msg void    OnCbnSelendokPri1();
        afx_msg void    OnCbnSelendokPri2();
        afx_msg void    OnCbnSelendokPri3();
        afx_msg void    OnCbnSelendokPri4();
        afx_msg void    OnCbnSelendokPri5();
        afx_msg void    OnCbnSelendokPri6();
        afx_msg void    OnCbnSelendokPri7();
        afx_msg void    OnCbnSelendokPri8();
        afx_msg void    OnCbnSelendokProcessPriority();
        afx_msg void    OnCbnSelendokTestLength();
        afx_msg LRESULT OnClickAll(WPARAM, LPARAM);
        afx_msg void    OnClose();
        afx_msg LRESULT OnConfigurationChange(WPARAM, LPARAM);
        afx_msg BOOL    OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct);
        afx_msg void    OnDestroy();
        afx_msg void    OnEditCopy();
        afx_msg void    OnEditCut();
        afx_msg void    OnEditDelete();
        afx_msg void    OnEditSelectall();
        afx_msg void    OnFileExit();
        afx_msg void    OnGUI(UINT id);
        afx_msg void    OnGetMinMaxInfo(MINMAXINFO* lpMMI);
        afx_msg void    OnHelpAbout();
        afx_msg void    OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
        afx_msg void    OnLButtonDown(UINT nFlags, CPoint point);
        afx_msg void    OnLButtonUp(UINT nFlags, CPoint point);
        afx_msg LRESULT OnLog(WPARAM, LPARAM);
        afx_msg void    OnLogClear();
        afx_msg void    OnLogPrint();
        afx_msg void    OnLogSave();
        afx_msg void    OnLogSaveas();
        afx_msg void    OnLogSetfont();
        afx_msg void    OnLogToEnd();
        afx_msg void    OnLogToNextError();
        afx_msg void    OnLogToPreviousError();
        afx_msg void    OnLogToTop();
        afx_msg void    OnMouseMove(UINT nFlags, CPoint point);
        virtual void    OnOK();
        afx_msg LRESULT OnProcessNotification(WPARAM, LPARAM);
        afx_msg LRESULT OnRequestPlot(WPARAM, LPARAM);
        afx_msg LRESULT OnRunAll(WPARAM, LPARAM);
        afx_msg BOOL    OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
        afx_msg LRESULT OnSetPos(WPARAM, LPARAM);
        afx_msg LRESULT OnSetupComplete(WPARAM, LPARAM);
        afx_msg void    OnSize(UINT nType, int cx, int cy);
        afx_msg LRESULT OnStartupFailure(WPARAM, LPARAM);
        afx_msg LRESULT OnStopped(WPARAM, LPARAM);
        afx_msg BOOL    OnToolTipNotify(UINT id, NMHDR * pNMHDR, LRESULT * pResult);
        DECLARE_MESSAGE_MAP()


protected: // Controls
        CButton          c_All;
        CSmartDropdown   c_Configurations;
        CButton          c_Copy;
        CButton          c_CopyAll;
        CButton          c_CopyPriorityClass;
        CStatic          c_CopyProgressFrame;
        CButton          c_CopyProgress;
        CStatic          c_CopyRect;
        CStatic          c_Frame;
        CArray<CGraph*, CGraph*> graphs;
        CGraph           c_Graph0; // The basic graphs[0] object
        CButton          c_Launch;
        CTraceList       c_Log;
        CPriority        c_PriorityDisplay[16];
        CStatic          c_PriorityCopyFrame;
        CProcessPriority c_ProcessPriority;
        CButton          c_ResetAll;
        CButton          c_Stop;
        CIDCombo         c_TestLength;

/*****************************************************************************
*                          The per-thread control block
*****************************************************************************/
protected:
#define MAX_CPUS    8

   class CThreadControl {
       public:
          CThreadControl();
          //****************************************************************
          // The following are public so the DoDataExchange in the parent
          // window can set them
          //****************************************************************
       public: 
          CStatic c_ThreadHandle;
          CStatic c_ThreadID;
          CButton c_Go;
          CPriorities c_Priority;
          CButton c_Affinity[MAX_CPUS];
          CStatic c_Mask;
          CProgressCtrl c_Progress;
          CStatic c_Time;

       public:
          void AddPlotData(CGraph & graph);
          BOOL CanGo();
          BOOL ChangeAffinityMask();
          BOOL ChangeThreadPriority();
          void EnableByMask(UINT_PTR m);
          void EnablePriority(BOOL m);
          void Finished(LPARAM t);
          DWORD_PTR GetAffinityMask();
          DWORD GetIterations() { return iterations; }
          int GetPriority();
          CString GetPriorityName();
          void InitControls(CWnd * p);
          void OnGo();
          void ResetControls();
          void ResetProgress();
          void RunWaiting();
          void SetAffinityMask(UINT_PTR m);
          void SetColor(COLORREF c);
          void SetIndex(int i) { index = i; }
          void SetIterations(DWORD n) { iterations = n; }
          void SetProgress(int n);
          void Stop() { running = FALSE; }
          void updateControls();
       protected:
          //****************
          // Graph information
          //****************
          DWORD start;
          CArray<POSINFO *, POSINFO *>graph;
          COLORREF linecolor;
          int iterations;
       public:
          THREADINFO GetThreadInfo();
       public:
          void ClearGraph();
          
          //****************
          // Fixed data
          //****************
       protected:
          int index;

          //****************
          // Thread data
          //****************
       protected:
          volatile BOOL running;
          CWinThread * thread;
          static UINT runthread(LPVOID v);
          void runthread();
          void DoNothing();
          int priority;
       public:
          BOOL HasThread();

          //****************
          // Logging
          //****************
       protected:
          CWnd * parent;
          void Log(TraceEvent *);
          //****************
          // Registry
          //****************
       protected:
          CString MakeRegistryKey();
          CString configuration;
       public:
          void LoadProfile();
          void SaveProfile();
          void SetConfiguration(const CString & s);

   };

protected:
   CThreadControl ThreadInfo[MAX_THREADS + 1]; // +1 to allow for the GUI thread to be configured
};




